@@ -13,7 +13,7 @@
  * See the GNU Library General Public License version 2 for more details
  *)
 
-(*i $Id: date.ml,v 1.15 2003/09/18 14:34:01 signoles Exp $ i*)
+(*i $Id: date.ml,v 1.24 2004/11/13 20:14:07 signoles Exp $ i*)
 
 (*S Introduction.
 
@@ -31,6 +31,8 @@ type day = Sun | Mon | Tue | Wed | Thu | Fri | Sat
 type month = 
     Jan | Feb | Mar | Apr | May | Jun | Jul | Aug | Sep | Oct | Nov | Dec
 
+type year = int
+
 type field = [ `Year | `Month | `Week | `Day ]
 
 (*S Exceptions. *)
@@ -80,10 +82,9 @@ let from_unixtm x =
   make (x.Unix.tm_year + 1900) (x.Unix.tm_mon + 1) x.Unix.tm_mday
 
 let today () = 
-  let today = Unix.gmtime (Unix.gettimeofday ()) in
-  let d = (* current day at GMT *)
-    from_unixtm today
-  and hour = Time_Zone.from_gmt () + today.Unix.tm_hour in
+  let today = Unix.gmtime (Unix.time ()) in
+  let d = (* current day at GMT *) from_unixtm today in
+  let hour = Time_Zone.from_gmt () + today.Unix.tm_hour in
   (* change the day according to the time zone *)
   if hour < 0 then begin
     assert (hour > - 13); 
@@ -115,7 +116,9 @@ let is_leap_year y =
 
 (*S Boolean operations on dates. *)
 
-let compare = (-)
+let compare = compare
+
+let equal = (==)
 
 let is_julian d = d < 2299161
 
@@ -153,7 +156,9 @@ let year d =
   let e = c - (1461 * d) / 4 in
   b * 100 + d - 4800 + ((5 * e + 2) / 153) / 10
 
-let day_of_week d = day_of_int ((d + 1) mod 7)
+let int_day_of_week d = (d + 1) mod 7
+
+let day_of_week d = day_of_int (int_day_of_week d)
 
 let day_of_year d = d - make (year d - 1) 12 31
 
@@ -210,9 +215,14 @@ module Period = struct
      Yep, [Pervasives.compare] correctly works. *)
   let compare = Pervasives.compare
 
+  let equal = (=)
+
   exception Not_computable
 
   let nb_days p = if p.y <> 0 || p.m <> 0 then raise Not_computable else p.d
+
+  let ymd p = p.y, p.m, p.d
+
 end
 
 (*S Arithmetic operations on dates and periods. *)
@@ -251,15 +261,30 @@ let same_calendar y1 y2 =
     else false
   in d mod 28 = 0 || aux
 
-let days_in_year y = if is_leap_year y then 366 else 365
+let days_in_year =
+  let days = [| 31; 59; 90; 120; 151; 181; 212; 243; 273; 304; 334; 365 |] in
+  fun ?(month=Dec) y ->
+    let m = int_of_month month in
+    let res = days.(m) in
+    if is_leap_year y && m > 0 then res + 1 else res
 
 let weeks_in_year y =
   let first_day = day_of_week (make y 1 1) in
   match first_day with
-    | Tue -> if is_leap_year y then 52 else 53
+    | Thu -> 53
     | Wed -> if is_leap_year y then 53 else 52
     | _   -> 52
 
+let week_first_last w y =
+  let d = make y 1 1 in
+  let d = d - d mod 7 in
+  let b = d + 7 * (w - 1) in
+  b, 6 + b
+
+let nth_weekday_of_month y m d n =
+  let first = make y (int_of_month m + 1) 1 in
+  first + int_of_day d - int_day_of_week first + (n - 1) * 7
+
 let century y = if y mod 100 = 0 then y / 100 else y / 100 + 1
 
 let millenium y = if y mod 1000 = 0 then y / 1000 else y / 1000 + 1
@@ -295,6 +320,18 @@ let easter y =
   let m = 3 + (l + 40) / 44 in
   make y m (l + 28 - 31 * (m / 4))
 
+let carnaval y = easter y - 48
+let mardi_gras y = easter y - 47
+let ash y = easter y - 46
+let palm y = easter y - 7
+let easter_friday y = easter y - 2
+let easter_saturday y = easter y - 1
+let easter_monday y = easter y + 1
+let ascension y = easter y + 39
+let withsunday y = easter y + 49
+let withmonday y = easter y + 50
+let corpus_christi y = easter y + 60
+
 (*S Exported Coercions. *)
 
 let to_unixtm d =
@@ -302,28 +339,49 @@ let to_unixtm d =
     Unix.tm_mday = day_of_month d; 
     Unix.tm_mon = int_month d - 1;
     Unix.tm_year = year d - 1900;
-    Unix.tm_wday = int_of_day (day_of_week d);
+    Unix.tm_wday = int_day_of_week d;
     Unix.tm_yday = day_of_year d - 1;
     Unix.tm_isdst = false }
 
 let jan_1_1970 = 2440588
 
-let to_unixfloat x = float_of_int (x - jan_1_1970)
+let to_unixfloat x = float_of_int (x - jan_1_1970) *. 86400.
+  (* do not replace [*.] by [*]: the result is bigger than [max_int] ! *)
 
-let from_unixfloat x = int_of_float x + jan_1_1970
+let from_unixfloat x = int_of_float (x /. 86400.) + jan_1_1970
 
-(* These coercions redefined those defined at the beginning of the module.
-   They respect ISO-8601. *)
+let to_business d =
+  let w = week d in
+  let y =
+    let y = year d in
+    match int_month d with
+      | 1 -> if w = 1 then y else y - 1
+      | 12 -> if w = 1 then y + 1 else y
+      | _ -> y
+  in
+  y, w, day_of_week d
 
 let int_of_day d = let n = int_of_day d in if n = 0 then 7 else n
+  (* Used by [from_business] *)
+
+let from_business y w d =
+  if w < 1 || w > weeks_in_year y then invalid_arg "from_business";
+  let first = make y 1 1 in
+  let first_day = int_day_of_week first in
+  let w = if first_day > 4 then w else w - 1 in
+  first + w * 7 + int_of_day d - first_day
+
+(* These coercions redefine those defined at the beginning of the module.
+   They respect ISO-8601. *)
+
+let int_of_day = int_of_day
 
 let day_of_int n = 
-  if n > 0 && n < 7 then day_of_int n
-  else if n = 7 then day_of_int 0
-  else raise (Invalid_argument "Not a day")
+  if n > 0 && n < 7 then day_of_int n 
+  else if n = 7 then day_of_int 0 
+  else invalid_arg "Not a day"
 
 let int_of_month m = int_of_month m + 1
 
 let month_of_int n =
-  if n > 0 && n < 13 then month_of_int (n - 1) 
-  else raise (Invalid_argument "Not a month")
+  if n > 0 && n < 13 then month_of_int (n - 1) else invalid_arg "Not a month"
