@@ -1,4 +1,4 @@
-(*i $Id: test_date.ml,v 1.9 2003/09/18 16:15:11 signoles Exp $ i*)
+(*i $Id: test_date.ml,v 1.14 2004/11/13 20:14:07 signoles Exp $ i*)
 
 Printf.printf "\nTests of Date:\n\n";;
 
@@ -41,22 +41,49 @@ test (same_calendar 1998 2009) "same calendar 1998 2009";;
 test (same_calendar 2003 2025) "same calendar 2003 2025";;
 test (days_in_year 2000 = 366) "days_in_year 2000";;
 test (days_in_year 1900 = 365) "days_in_year 1900";;
+test (days_in_year ~month:Jan 2000 = 31) "days_in_year Jan 2000";;
+test (days_in_year ~month:Feb 2000 = 60) "days_in_year Feb 2000";;
+test (days_in_year ~month:Jan 2000 = 31) "days_in_year Jan 2000";;
+test (days_in_year ~month:Mar 1900 = 90) "days_in_year Mar 1900";;
 test (weeks_in_year 2000 = 52) "weeks_in_year 2000";;
 test (weeks_in_year 2020 = 53) "weeks_in_year 2020";;
-test (weeks_in_year 1991 = 53) "weeks_in_year 1991";;
-test (weeks_in_year 1999 = 52) "weeks_in_year 1991";;
+test (weeks_in_year 1991 = 52) "weeks_in_year 1991";;
+test (weeks_in_year 1999 = 52) "weeks_in_year 1999";;
 test (century 2000 = 20) "century 2000";;
 test (century 2001 = 21) "century 2001";;
 test (millenium 2000 = 2) "millenium 2000";;
 test (millenium 2001 = 3) "millenium 2001";;
 test (easter 2003 = make 2003 4 20) "Paques 2003";;
-test (to_unixfloat (make 1970 1 1) = 0.) "to_unixfloat";;
-test (from_unixfloat 0. = make 1970 1 1) "from_unixfloat";;
+test (to_unixfloat (make 1970 1 1) = 0.) "to_unixfloat 1 Jan 1970";;
+test (from_unixfloat 0. = make 1970 1 1) "from_unixfloat 1 Jan 1970";;
+test (to_unixfloat (make 2004 11 13) = 1100304000.) "to_unixfloat";;
+test (from_unixfloat 1100304000. = make 2004 11 13) "to_unixfloat";;
 test (from_unixtm (to_unixtm (make 2003 7 16)) = make 2003 7 16) 
   "from_unixtm to_unixtm = id";;
-
 test (Period.nb_days (Period.make 0 0 6) = 6) "Period.nb_days ok";;
 test_exn (lazy (Period.nb_days (Period.make 1 0 0))) "Period.nb_days ko";;
+test (week_first_last 21 2004 = (make 2004 5 17, make 2004 5 23)) 
+  "week_beggining_end";;
+test (Period.ymd (Period.make 1 2 3) = (1, 2, 3)) "Period.ymd";;
+test (nth_weekday_of_month 2004 Oct Thu 5 = make 2004 10 28) 
+  "nth_weekday_of_month";;
+
+(* to_business *)
+test (to_business (make 2003 1 1) = (2003, 1, Wed)) "to_business 1";;
+test (to_business (make 2003 12 31) = (2004, 1, Wed)) "to_business 2";;
+test (to_business (make 2002 12 31) = (2003, 1, Tue)) "to_business 3";;
+test (to_business (make 2005 1 1) = (2004, 53, Sat)) "to_business 4";;
+test (to_business (make 2004 12 31) = (2004, 53, Fri)) "to_business 5";;
+test (to_business (make 2006 1 1) = (2005, 52, Sun)) "to_business 6";;
+(* from_business *)
+test (from_business 2003 1 Wed = make 2003 1 1) "from_business 1";;
+test (from_business 2004 1 Wed = make 2003 12 31) "from_business 2";;
+test (from_business 2003 1 Tue = make 2002 12 31) "from_business 3";;
+test (from_business 2004 53 Sat = make 2005 1 1) "from_business 4";;
+test (from_business 2004 53 Fri = make 2004 12 31) "from_business 5";;
+test (from_business 2005 52 Sun = make 2006 1 1) "from_business 6";;
+test_exn (lazy (from_business 2005 0 Sun)) "from_business_bad 1";;
+test_exn (lazy (from_business 2005 53 Sun)) "from_business_bad 2";;
 
 let ok = nb_ok ();;
 let bug = nb_bug ();;
